<?php

defined('_VALID_CALL') or die('Direct Access is not allowed.');

/**
 * Class callback_unzer_direct will be called by xt when callback messages will come for unzer_direct server
 *
 */
class callback_unzer_direct extends callback {

    /**
     * @var
     * QuickPay private key
     */
    private
        $private_key;

    /**
     * @var
     *  request received from quickpay server
     */
    private $request_body;

    /**
     * @param $base
     * @param $private_key
     * @return false|string
     */
    private function sign($base, $private_key) {
        return hash_hmac("sha256", $base, $private_key);
    }

    /**
     * converts quickpay_id to orders_id in xt:commerce format
     * @param $quickpay_id ID from QuickPay payment
     * @return $orders_id for xt order
     */
    private function ConvertToOrdersId($quickpay_id) {
        global $db;

        // getting orders_id for quickpay_id
        $rs  = $db->Execute("SELECT	orders_id FROM " . TABLE_ORDERS . " WHERE quickpay_id = '" . $quickpay_id . "'");

        if ($rs->RecordCount()!=0) {
           return $rs->fields['orders_id'];
        }
        return false;
    }

    /**
     * Analizes response from QuickPay and updates xt order status
     * @return bool
     */
    private function SetOrderStatus() {

        $QuickPayResponse = json_decode($this->request_body);
        $operations_count = count($QuickPayResponse->operations);
        $last_operation = $QuickPayResponse->operations[$operations_count - 1];

        if ($last_operation->qp_status_code == '20000') {
            switch ($last_operation->type) {
                case 'authorize':
                    // we don't register this action in xt order
                    return true;
                case 'refund':
                    if ($QuickPayResponse->balance == 0) {
                        $new_order_status = UNZER_DIRECT_ORDER_STATUS_REFUNDED;
                    } else {
                        $new_order_status = UNZER_DIRECT_ORDER_STATUS_PARTIALLY_REFUNDED;
                        $amount = round($last_operation->amount / 100, 2);
                    }
                    break;
                case 'capture':
                    if ($QuickPayResponse->balance == $QuickPayResponse->link->amount) {
                        $new_order_status = UNZER_DIRECT_ORDER_STATUS_COMPLETED;
                    } else {
                        $new_order_status = UNZER_DIRECT_ORDER_STATUS_PENDING;
                        $amount = round($last_operation->amount / 100, 2);
                    }
                    break;
                default:
                    $new_order_status = UNZER_DIRECT_ORDER_STATUS_FAILED;
                }
        } else {
            $new_order_status = UNZER_DIRECT_ORDER_STATUS_FAILED;
        }

        // $callback_message will be shown in order status-> IPN message
        $callback_message = $last_operation->type.':'.$last_operation->qp_status_msg;
        if (!empty($amount)) {
            $callback_message.=' Amount: '.$amount;
        }
        $callback_message = substr($callback_message,0,255);

        // update order status
        // $new_order_status will be shown in order status in XT admin, $QuickPayResponse->id willbe shown as IPN ID
        $this->_updateOrderStatus($new_order_status,false, $QuickPayResponse->id,$callback_message);
        return true;
    }

    /**
     * Checks signature, analizes quickpay request and updates xt order info
     * @return bool
     */
    private function _callbackProcess() {

        $this->private_key=trim(UNZER_DIRECT_PRIVATE_KEY);
        $this->request_body = file_get_contents("php://input");

        // validate signature
        $success = $this->CheckSignature();
        if (!$success) return false;

        // convert and validate orders_id
        $QuickPayResponse=json_decode($this->request_body);
        $this->orders_id=$this->ConvertToOrdersId($QuickPayResponse->id);

        if (!$this->orders_id) return false;

        $success=$this->SetOrderStatus();
        return $success;
    }

    /**
     * checks signature of QuickPay request
     * @return bool
     */
    private function CheckSignature() {

        $checksum     = $this->sign($this->request_body, $this->private_key);

        if ($checksum == $_SERVER["HTTP_QUICKPAY_CHECKSUM_SHA256"]) {
            // Request is authenticated
            return true;
        } else {
            return false;
        }
    }

    /**
     * this function will be called by xt when callback message will arrive for quickpay module
     */
    function process() {
        $success = $this->_callbackProcess();
        if ($success) {
            header("HTTP/1.0 200 OK");
        } else {
            header('HTTP/1.0 404 Not Found');
        }
    }
}