<?php
defined('_VALID_CALL') or die('Direct Access is not allowed.');

use unzer_direct\QuickPayBase;

require_once _SRV_WEBROOT . _SRV_WEB_PLUGINS . 'unzer_direct/classes/base/class.QuickPayBase.php';


/**
 * Class unzer_direct will be called by xt:commerce when user will submit order
 */

class unzer_direct
{
// these fields required by xt:commerce , don't delete!!!
    var $data = array();
    var $external = true;
    var $version = '1.0';
    var $iframe = false;

    protected $new_order_id;  //order_id in QucikPay format
    protected $client;

    /**
     * quickpay constructor.
     */
    function __construct ()
    {
        global $xtLink;

        $this->RETURN_URL = $xtLink->_link(array('page' => 'checkout', 'paction' => 'payment_process'));
        $this->ERROR_URL = $xtLink->_link(array('page' => 'checkout', 'paction' => 'payment', 'params' => 'error=ERROR_PAYMENT'));
        $this->CANCEL_URL = $xtLink->_link(array('page' => 'checkout', 'paction' => 'payment'));
        $this->NOTIFY_URL = $xtLink->_link(array('page' => 'callback', 'paction' => 'unzer_direct'));
        $this->NOTIFY_URL=str_replace('xtAdmin/', '',$this->NOTIFY_URL);
    }

    /**
     * @param $orders_id int orders_id from xt:commerce
     * @return int order_id with lenth from 4 to 20 chars as needed for QuickPay
     */
    function PrepareOrdersID($orders_id) {
        $order_id=(string)$orders_id;
        while (strlen($order_id)<4) {
            $order_id = '0'.$order_id;
        }
        if (strlen($order_id)>20) {
            $order_id=substr($order_id,0,20);
        }
        return $order_id;
    }

    /**
     * converts ISO 3166-1 alpha-2 country code to ISO 3166-1 alpha-3 code (needed by QuickPay)
     * @param string $alpha2_country_code string with 2 letters country code
     */
    function ConvertCountryCodeToAlpha3($alpha2_country_code) {
        require _SRV_WEBROOT . _SRV_WEB_PLUGINS . 'unzer_direct/classes/base/country_array.php';

        foreach ($countries as $country) {
            if ($country['alpha2']==$alpha2_country_code)
                return $country['alpha3'];
        }
        return '';
    }

    /**
     * converts orders_id from xtcommerce format to quickpay_id
     * @return $quickpay_id
     */
    private function ConvertToQuickPayID($orders_id) {
        global $db;

        // getting quickpay_id for orders_id
        $rs  = $db->Execute("SELECT	quickpay_id FROM " . TABLE_ORDERS . " WHERE orders_id = '" . $orders_id . "'");
        if ($rs->RecordCount()!=0) {
            return $rs->fields['quickpay_id'];
        }
        return false;
    }

    /**
     * full refund of specific order
     *  it will be called by hook when order status changed to "Refunded" in xt order properties
     * @param $orders_id
     */
    function Refund($orders_id) {
        global $xtLink;

        //Initialize client
        try {
            $api_key=trim(UNZER_DIRECT_API_KEY);
            $additional_headers = array('QuickPay-Callback-Url: '.$this->NOTIFY_URL);
            $client = new QuickPayBase(":{$api_key}",$additional_headers);
        } catch (Exception $e) {
            return false;
        }

        // Get QuickPay id
        $quickpay_id=$this->ConvertToQuickPayID($orders_id);
        $payment = $client->request->get('/payments/'.$quickpay_id)->asArray();
        $amount=$payment['balance'];

        // Create refund request
        $payment_params = array (
            'id' => $quickpay_id,
            'amount' => $amount,
        );

        $payment = $client->request->post('/payments/'.$quickpay_id.'/refund', $payment_params);

        //Determine if payment was created successfully
        $status = $payment->httpStatus();

        if (($status != 201) && ($status != 202)) {
            $xtLink->_redirect($this->ERROR_URL);
        }
    }

    /**
     *  builds payment params for QuickPay and creates payment on the server
     *  this payment can be used later to create payment link
     * @param $order
     * @return \QuickPay\API\Response
     */
    function CreatePayment($order) {

        //Initialize client
        try {
            $api_key=trim(UNZER_DIRECT_API_KEY);
            $this->client = new QuickPayBase(":{$api_key}");
        } catch (Exception $e) {
            global $xtLink;
            $xtLink->_redirect($this->ERROR_URL);
        }

        //Create payment
        if (empty($this->new_order_id)) {
            $order_id=$this->PrepareOrdersID($order->order_data['orders_id']);
        } else {
            $order_id=$this->new_order_id;
        }

        $payment_params = array (
            'order_id' => $order_id,
            'currency' => $order->order_data['currency_code'],
            'shipping_address[name]' => $order->order_data['delivery_firstname'].' '.$order->order_data['delivery_lastname'],
            'shipping_address[street]' => $order->order_data['delivery_street_address'],
            'shipping_address[house_number]' => $order->order_data['delivery_street_address'],
            'shipping_address[att]' => $order->order_data['delivery_address_addition'],
            'shipping_address[city]' => $order->order_data['delivery_city'],
            'shipping_address[zip_code]' => $order->order_data['delivery_postcode'],
            'shipping_address[company_name]' => $order->order_data['delivery_company'],
            'shipping_address[phone_number]' => $order->order_data['delivery_phone'],
            'shipping_address[mobile_number]' => $order->order_data['delivery_mobile_phone'],
            'invoice_address[name]' => $order->order_data['billing_firstname'].' '.$order->order_data['billing_lastname'],
            'invoice_address[company_name]' => $order->order_data['billing_company'],
            'invoice_address[street]' => $order->order_data['billing_street_address'],
            'invoice_address[att]' => $order->order_data['billing_address_addition'],
            'invoice_address[city]' => $order->order_data['billing_city'],
            'invoice_address[house_number]' =>$order->order_data['billing_street_address'], // ? which field from xt to associate ?
            'invoice_address[phone_number]' => $order->order_data['billing_phone'],
            'invoice_address[mobile_number]' => $order->order_data['billing_mobile_phone'],
            'invoice_address[zip_code]' => $order->order_data['billing_postcode'],
            'shipping[company]' => $order->order_data['shipping_name'],
            'shipping[amount]' => (int)($order->order_data['shipping_price']*100),
            'shopsystem[version]' => 'xt:Commerce '._SYSTEM_VERSION,
            'shopsystem[name]' =>_STORE_NAME,
        );

        $a=$this->ConvertCountryCodeToAlpha3($order->order_data['delivery_country_code']);
        if (!empty($a)) {
            $payment_params['shipping_address[country_code]'] =$a;
        }

        $a=$this->ConvertCountryCodeToAlpha3($order->order_data['billing_country_code']);
        if (!empty($a)) {
            $payment_params['invoice_address[country_code]'] =$a;
        }

        // adding basket items
        $i=0;
        foreach ($order->order_products as $product) {
            $item['item_name']=$product['products_name'];
            $item['qty']=(int)$product['products_quantity'];
            $item['item_no']=$product['products_id'];
            $price=$product['products_price'];
            $item['item_price']=(int)($price['plain']*100);
            //$item['vat_rate']=$product['products_tax_rate'];   //doesn't accepted by QuickPay
            $item['vat_rate']='';
            $payment_params['basket['.$i.']']=$item;
            $i++;
        }

        // request to QuickPay to create new payment
        return $this->client->request->post('/payments', $payment_params);
    }

    /**
     * checks if order_id is valid for QuickPay
     * returns false if there are some errors in QuickPay response for order_id field e.g. duplicates
     * or incorrect length
     *
     * @param $paymentObject
     * @return bool
     */
    function isOrderIDCorrect($paymentObject){
        if (array_key_exists('order_id',$paymentObject->errors))
            return false;

        return true;
    }

    /**
     * creates PaymentLink URL for QuickPay using xt:commerce order
     * @param $order
     * order information
     * @return string
     * returns link url or false
     */
    function CreatePaymentLink($order) {

        global $xtLink, $db;

        $MaxAttemptsNumber=20; // max number of times to try different order_id
        $AttemptsNumber=0;
        // creation of payment, if dublicate order_id - generating new order_id
        do {
            $payment = $this->CreatePayment($order);
            $paymentObject = $payment->asObject();
            // Determine if payment was created successfully
            $status = $payment->httpStatus();
            if ( $status!= 201) {
                if (!$this->isOrderIDCorrect($paymentObject)) {
                    $this->new_order_id=time();
                    $AttemptsNumber++;
                }
                else {
                    $xtLink->_redirect($this->ERROR_URL);
                }
            }
        }
        while ( ($status!= 201) && ($AttemptsNumber<=$MaxAttemptsNumber) );

        //Construct url to create payment link
        $endpoint = sprintf("/payments/%s/link", $paymentObject->id);

        // adding qucikpay id to orders table
        $db->Execute("UPDATE " . TABLE_ORDERS . " SET quickpay_id=?"." WHERE orders_id=?", array($paymentObject->id, $order->oID));


        // choose if capture payment automatically, depending on plugin options
        $auto_capture=(constant('UNZER_DIRECT_CAPTURE')==1) ? 'true' : 'false';

        //Issue a put request to create payment link
        $link = $this->client->request->put($endpoint, [
            'amount' => round($order->order_total['total']['plain']*100, 0), //amount in cents
            'language' => $order->order_data['language_code'],
            'continue_url' => $this->RETURN_URL,
            'cancel_url' => $this->CANCEL_URL,
            'callback_url' => $this->NOTIFY_URL,
            'auto_capture' => $auto_capture,
            'customer_email' =>$order->order_data['customers_email_address'],
        ]);

        //Determine if payment link was created succesfully
        if ($link->httpStatus() === 200) {
            //Get payment link url
            return $link->asObject()->url;
        } else {
            $xtLink->_redirect($this->ERROR_URL);
            return false;
        }
    }

    function pspSuccess ()
    {
        return true;
    }

    /**
     *
     * @param array $processed_data
     * array passed to this function by xt:commerce with order info
     *
     * @return string
     * returns URL on QuickPay site with prepared payment information
     */
    function pspRedirect ($processed_data = array())
    {
        global $xtLink, $db;

        $orders_id = (int)$_SESSION['last_order_id'];

        $rs = $db->Execute("SELECT customers_id FROM " . TABLE_ORDERS . " WHERE orders_id=?", array($orders_id));

        $order = new order($orders_id, $rs->fields['customers_id']);
        $order->order_data['shipping_price']=$processed_data['shipping_price'];

        $payment_link = $this->CreatePaymentLink($order);

        if(empty($payment_link))
        {
            $xtLink->_redirect($this->ERROR_URL);
        }
        return $payment_link;
    }
}
